///////////////////////////////////////////////////////////////
//	Define the CWnDrv Multi Thread Video Motion Detection Class 
///////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "CWnDrv.h"

#include <sys/timeb.h>
#include <direct.h>

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Video Motion Detection Class Construction and initialization
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
CWnDrv::CWnDrv(CWnd* pWnd,
			           WORD  wHWCardNo,
			           BYTE* lpImage,
			           BITMAPINFO* lpDIBHdr)
{
	m_hDatProcThread = NULL ;

	if( !wnwd_SystemInit(wHWCardNo,lpImage,lpDIBHdr) ) 
		PRINTF("(WnWizard)Initializing Error" );
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Video Motion Detection Class Destruction and Clean up
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
CWnDrv::~CWnDrv()
{
	wnwd_SystemFinish();

	m_hDatProcThread = NULL ;

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Create This Video Motion Detection Thread
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BOOL CWnDrv::CreateThread( DWORD dwCreateFlags, 
							   UINT nStackSize, 
					           LPSECURITY_ATTRIBUTES lpSecurityAttrs )
{

    DWORD dwThreadID;
	m_DatProcMilliSecond = 0;
    m_hDatProcThread = ::CreateThread(
                    NULL,
                    0,
                    (unsigned long (__stdcall *)(void *))DatProcThreadInitialProc,
                    this,
                    CREATE_SUSPENDED,
                    &dwThreadID);

	if( m_hDatProcThread )
	::SetThreadPriority(m_hDatProcThread , THREAD_PRIORITY_HIGHEST);
	return TRUE ;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Start This Video Motion Detection Thread
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DWORD CWnDrv::ResumeThread()
{
	DWORD ret =0;

	m_DatProcMilliSecond = 80 ;  
	
	if( !m_hDatProcThread  ) CreateThread(CREATE_SUSPENDED); 

	ret = ::ResumeThread( m_hDatProcThread );
	
	return ret ;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Stop This Video Motion Detection Thread
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DWORD CWnDrv::SuspendThread()
{
	DWORD ret =0;

	m_DatProcMilliSecond = 0 ;  

	ret |= ::SuspendThread( m_hDatProcThread );

	return ret ;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Delete This Video Motion Detection Thread
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::KillThread()
{

	m_DatProcMilliSecond = 0 ;

	if (m_hDatProcThread) {

        ::WaitForSingleObject(m_hDatProcThread,1);
		#define WARN_EXIT_CODE 0xfff+3 
		::TerminateThread( m_hDatProcThread,  WARN_EXIT_CODE );
        ::CloseHandle(m_hDatProcThread);
        m_hDatProcThread = NULL;
    }

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Begin This Video Motion Detection Thread to Monitor the Video Motion
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::DatProcThreadProc(void)
{

    for (;;)  
	{
		::WaitForSingleObject(m_hDatProcThread, m_DatProcMilliSecond); 

	    if( wnwd_VImgIsMotify() )
		{
			m_WarnStatus = TRUE;
			// warning 
			if(m_bIsWarn )
			MessageBeep( MB_ICONEXCLAMATION  );
			//write BMP 
			if(m_bIsSave ) 
			{
			char pTemp[100] , pTempA[100] ; 
			strcpy(pTempA, "C:\\");
			sprintf(pTemp,"MotionDetect%d" , m_wHWCurrentCardNum) ;
			strcat(pTempA, pTemp);
			wnwd_SaveBMP((char *)((const char*)pTempA));
			}
		}
		else
		m_WarnStatus = FALSE;
		
    }
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// initialize This Video Motion Detection Thread Parameter with the Hardware 
// Capture Card Number, Real Time Video Frame Image address and 
// Real Time Video Frame Image Format address
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BOOL CWnDrv::wnwd_SystemInit (WORD wHWCurrentCardNum,
								  BYTE* lpImage,
      		                      BITMAPINFO* lpDIBHdr)
{
	
	m_wHWCurrentCardNum = wHWCurrentCardNum; 

	m_pBtImage= lpImage ; 
	m_pDIBHdr = lpDIBHdr; 

	m_pBtImageBak = NULL;

	m_pYUVSample = m_pYUV = NULL  ;
	m_bIsSave = m_bIsWarn = FALSE ;
	m_WarnStatus = FALSE;
	m_iVGABits= (int)m_pDIBHdr->bmiHeader.biBitCount/8;

    wnwd_SetCmpStepXY(8,8);
    wnwd_SetCmpStepYUV(16,8,8);
    wnwd_SetCmpPercent(0.10);
	wnwd_CmpInit();
	return TRUE;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// cleanup This Video Motion Detection Thread memory 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_SystemFinish()
{	
	FREE(m_pYUV) ;
	FREE(m_pYUVSample) ; 
	FREE(m_pBtImageBak) ; 

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// set This Video Motion Detection Thread X,Y Direction Sample Point Step Number
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_SetCmpStepXY( WORD wStepX ,WORD wStepY )
{	
	if( wStepX < 4 ) wStepX = 4; 
	if( wStepY < 4 ) wStepY = 4; 

	m_wStepX = wStepX; m_wStepY = wStepY; 

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// set This Video Motion Detection Thread YUV Comparison value
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_SetCmpStepYUV(BYTE cStepY ,BYTE cStepU,BYTE cStepV )
{	
	if( cStepY < 4 ) cStepY = 4; 
	if( cStepU < 4 ) cStepU = 4; 
    if( cStepV < 4 ) cStepV = 4; 

	m_cStepY = cStepY; m_cStepU = cStepU; m_cStepV  = cStepV  ; 

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// set This Video Motion Detection Thread Comparison Percent Number
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_SetCmpPercent( double dPercent )
{	
    #define MIN_PERCENT 0.0001 
	if( dPercent < MIN_PERCENT ) dPercent = MIN_PERCENT ; 

	m_dPercent=dPercent; 

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// initialize This Video Motion Detection Thread memory 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_CmpInit()
{	
	FREE(m_pYUV) ;
	FREE(m_pYUVSample) ; 
	FREE(m_pBtImageBak) ; 

	m_lYUVSize = (DWORD)(m_lImgWid/m_wStepX)*
		         (DWORD)(m_lImgHei/m_wStepY)*3; 

	m_lYUVSampleSize = (DWORD)(m_lYUVSize*m_dPercent) ;
#define MIN_SAMPLESIZE 8
	if( m_lYUVSampleSize < MIN_SAMPLESIZE) m_lYUVSampleSize = MIN_SAMPLESIZE;

	ALLOC(m_pYUV      ,BYTE,m_lYUVSize+3) ; 
	ALLOC(m_pYUVSample,BYTE,m_lYUVSize+3) ; 

	memset(m_pYUV      , 0x00, m_lYUVSize+3) ; 
	memset(m_pYUVSample, 0x00, m_lYUVSize+3) ; 
	ALLOC(m_pBtImageBak,BYTE,m_lVImgSize+1) ; 

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Set This Video Capture Card Video Motion Detection Parameter 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_IsSetting()
{	
	wnwd_CmpInit();
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Set This Video Capture Card Video Motion Detection Saving Status  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_IsSave(BOOL bIsSave)
{	
	m_bIsSave = bIsSave;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Set This Video Capture Card Video Motion Detection Warning Status 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_IsWarn(BOOL bIsWarn)
{	
	m_bIsWarn = bIsWarn;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Get This Video Capture Card Video Motion Detection Status
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BOOL CWnDrv::wnwd_GetCmpStatus()
{	
	return m_WarnStatus;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Save This Video Capture Card Real Time Video Frame Image to a BMP File
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BOOL CWnDrv::wnwd_SaveBMP(char *FileName)
{ 

	if( !m_pDIBHdr ) return FALSE ;

	BITMAPFILEHEADER bmpHeader ;
	BITMAPINFO       bmpDIBHdr = *m_pDIBHdr ;
	RGBQUAD          *bmiColors = m_pDIBHdr->bmiColors ;
	int              iVGABits  = m_iVGABits ;

	memcpy(m_pBtImageBak, m_pBtImage,m_lVImgSize);

	FILE *fp ;
	char name[256];
	LONG FileLen, HeadOff ; 
    LONG  ImgHei  , ImgWid , i , Off ; 

    sprintf(name ,"%s.BMP", FileName);
    fp = fopen(name ,"wb" ) ;
   
    if( ! fp) {  char str[256]; 
    sprintf(str," Can't Write File %s ", FileName ) ; 
    MessageBox(NULL,TEXT(str) ,TEXT("Error"),  MB_OK ) ;  return FALSE ; }
  
	HeadOff =  sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
	if( iVGABits ==1) HeadOff +=COLOR_NUM*sizeof(RGBQUAD) ;
    FileLen =  bmpDIBHdr.bmiHeader.biSizeImage + HeadOff ;

	bmpHeader.bfType = 'MB';
	bmpHeader.bfSize = FileLen;
	bmpHeader.bfReserved1 = 0;
	bmpHeader.bfReserved2 = 0;
	bmpHeader.bfOffBits = HeadOff;

	ImgHei = bmpDIBHdr.bmiHeader.biHeight ;
	ImgWid = bmpDIBHdr.bmiHeader.biWidth*iVGABits ; 
	Off    = 0L ;

	fwrite( &bmpHeader, sizeof(BITMAPFILEHEADER) , 1  ,fp ) ;
	fwrite( &bmpDIBHdr, sizeof(BITMAPINFOHEADER) , 1  ,fp ) ;

	if(iVGABits==1) 
	fwrite(bmiColors,sizeof(RGBQUAD) ,COLOR_NUM, fp) ;

	for(i = 0 ; i < ImgHei ; i++ , Off += ImgWid) 
	fwrite( &m_pBtImageBak[Off] , 1 ,ImgWid ,fp ) ;
    fclose(fp) ;

    return TRUE ; }

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Change The Image Format From RGB to YUV
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_ChangeRGBtoYUV( unsigned char R, 
                                          unsigned char G,
						                  unsigned char B,
						                  unsigned char *Y,
						                  unsigned char *U,
						                  unsigned char *V)
{
 #define Cr  U 
 #define Cb  V 

	*Y  = (unsigned char)(  0.2569*R +0.5044*G +0.0978*B +16 );
	*Cr = (unsigned char)(  0.4392*R -0.3679*G -0.0713*B +128);
	*Cb = (unsigned char)( -0.1484*R -0.2909*G +0.4393*B +128);

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Change The Image Format From YUV to RGB
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_ChangeYUVtoRGB( unsigned char Y, 
                                       unsigned char U, 
						               unsigned char V, 
						               unsigned char *R, 
						               unsigned char *G, 
						               unsigned char *B)
{

 #define Cr  U 
 #define Cb  V 

 *R = (unsigned char)( 1.164*(Y-16) + 1.596*(Cr-128) )  ;
 *G = (unsigned char)( 1.164*(Y-16) - 0.813*(Cr-128) - 0.391*(Cb-128) )  ;
 *B = (unsigned char)( 1.164*(Y-16) + 2.018*(Cb-128) )  ;

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Monitor RGB15 Image Sample Point and change to YUV Format
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_RGB15_toYUV(  BYTE *lpRGB, BYTE *lpYUV,
	                                DWORD wWidth , DWORD wHeight ,
				                    WORD wStepX  , WORD wStepY ) 
{

  DWORD i, j ;
  RGBDATA lp15Data ; BYTE R,G,B,Y,U,V ;
  DWORD Off, PosRGB , PosRGBYUV = 0L;

  wWidth =(DWORD)(wWidth<<1) ; wStepX =(WORD)(wStepX<<1) ;

  for( i= wStepY,Off=0L; i < wHeight-wStepY ; i+=wStepY ,Off += wWidth )
  {
	  for( j= wStepX ; j < wWidth-wStepX ; j+=wStepX)
	  { 
		  PosRGB = Off+j  ; 
		  RRGBDATA(lp15Data) = MAKESHORT(lpRGB[PosRGB+1], lpRGB[PosRGB] );
          R = RR15(lp15Data); G = RG15(lp15Data); B = RB15(lp15Data);

		  wnwd_ChangeRGBtoYUV( R,G,B,&Y,&U,&V) ; 

		  lpYUV[ PosRGBYUV ] = Y ; PosRGBYUV++ ;
		  lpYUV[ PosRGBYUV ] = U ; PosRGBYUV++ ;
		  lpYUV[ PosRGBYUV ] = V ; PosRGBYUV++ ;

	  }
  }

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Monitor RGB16 Image Sample Point and change to YUV Format
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_RGB16_toYUV(  BYTE *lpRGB, BYTE *lpYUV,
	                                DWORD wWidth , DWORD wHeight ,
				                    WORD wStepX  , WORD wStepY ) 
{

  DWORD i, j ;
  RGBDATA lp16Data ; BYTE R,G,B,Y,U,V ;
  DWORD Off, PosRGB , PosRGBYUV = 0L;

  wWidth =(DWORD)(wWidth<<1) ; wStepX =(WORD)(wStepX<<1) ;

  for( i= wStepY,Off=0L; i < wHeight-wStepY ; i+=wStepY ,Off += wWidth )
  {
	  for( j= wStepX ; j < wWidth-wStepX ; j+=wStepX)
	  { 
		  PosRGB = Off+j  ;  
		  RRGBDATA(lp16Data) = MAKESHORT(lpRGB[PosRGB+1], lpRGB[PosRGB] );
          R = RR16(lp16Data); G = RG16(lp16Data); B = RB16(lp16Data);

		  wnwd_ChangeRGBtoYUV( R,G,B,&Y,&U,&V) ; 

		  lpYUV[ PosRGBYUV ] = Y ; PosRGBYUV++ ;
		  lpYUV[ PosRGBYUV ] = U ; PosRGBYUV++ ;
		  lpYUV[ PosRGBYUV ] = V ; PosRGBYUV++ ;

	  }
  }

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Monitor RGB24 Image Sample Point and change to YUV Format
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_RGB24_toYUV(  BYTE *lpRGB, BYTE *lpYUV,
	                                DWORD wWidth , DWORD wHeight ,
				                    WORD wStepX  , WORD wStepY ) 
{

  DWORD i, j ;
  BYTE Y,U,V ;
  DWORD Off, PosRGB , PosRGBYUV = 0L;

  wWidth =(DWORD)(wWidth*3) ; wStepX =(WORD)(wStepX*3) ;

  for( i= wStepY,Off=0L; i < wHeight-wStepY ; i+=wStepY ,Off += wWidth )
  {
	  for( j= wStepX ; j < wWidth-wStepX ; j+=wStepX)
	  { 
		  PosRGB = Off+j  ;  
          #define R24  lpRGB[PosRGB  ] 
          #define G24  lpRGB[PosRGB+1] 
		  #define B24  lpRGB[PosRGB+2] 
		  wnwd_ChangeRGBtoYUV( R24,G24,B24,&Y,&U,&V) ; 

		  lpYUV[ PosRGBYUV ] = Y ; PosRGBYUV++ ;
		  lpYUV[ PosRGBYUV ] = U ; PosRGBYUV++ ;
		  lpYUV[ PosRGBYUV ] = V ; PosRGBYUV++ ;

	  }
  }

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Monitor RGB32 Image Sample Point and change to YUV Format
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void CWnDrv::wnwd_RGB32_toYUV(  BYTE *lpRGB, BYTE *lpYUV,
	                                DWORD wWidth , DWORD wHeight ,
				                    WORD wStepX  , WORD wStepY ) 
{

  DWORD i, j ;
  BYTE Y,U,V ;
  DWORD Off, PosRGB , PosRGBYUV = 0L;

  wWidth =(DWORD)(wWidth*4) ; wStepX =(WORD)(wStepX*4) ;

  for( i= wStepY,Off=0L; i < wHeight-wStepY ; i+=wStepY ,Off += wWidth )
  {
	  for( j= wStepX ; j < wWidth-wStepX ; j+=wStepX)
	  { 
		  PosRGB = Off+j  ;  
          #define R32  lpRGB[PosRGB  ] 
          #define G32  lpRGB[PosRGB+1] 
		  #define B32  lpRGB[PosRGB+2] 
		  wnwd_ChangeRGBtoYUV( R32,G32,B32,&Y,&U,&V) ; 

		  lpYUV[ PosRGBYUV ] = Y ; PosRGBYUV++ ;
		  lpYUV[ PosRGBYUV ] = U ; PosRGBYUV++ ;
		  lpYUV[ PosRGBYUV ] = V ; PosRGBYUV++ ;

	  }
  }

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	Compare the realtime(m_pYUV) YUV value to the saved(m_pYUVSample) 
//  YUV value and saved the new value into the old one if motion detect
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BOOL CWnDrv::wnwd_VImgIsMotify()
{
	DWORD i , Count =0L;

	memcpy(m_pBtImageBak, m_pBtImage,m_lVImgSize);

	switch(m_iVGABits)
	{
	case CF_RGB15 : wnwd_RGB15_toYUV(m_pBtImageBak    , m_pYUV,
	                                 m_lImgWid , m_lImgHei, 
					                 m_wStepX  , m_wStepY ) ; break ;

	case CF_RGB16 : wnwd_RGB16_toYUV(m_pBtImageBak    , m_pYUV,
	                                 m_lImgWid , m_lImgHei, 
					                 m_wStepX  , m_wStepY ) ; break ;

	case CF_RGB24 : wnwd_RGB24_toYUV(m_pBtImageBak    , m_pYUV,
	                                 m_lImgWid , m_lImgHei, 
					                 m_wStepX  , m_wStepY ) ; break ;

	case CF_RGB32 : wnwd_RGB32_toYUV(m_pBtImageBak    , m_pYUV,
	                                 m_lImgWid , m_lImgHei, 
					                 m_wStepX  , m_wStepY ) ; break ;
	}

    #define Yi  m_pYUV[i]
    #define Ui  m_pYUV[i+1]
    #define Vi  m_pYUV[i+2]
    #define Ys  m_pYUVSample[i]
    #define Us  m_pYUVSample[i+1]
    #define Vs  m_pYUVSample[i+2]

	for(i=0 ; i < m_lYUVSize; i+=3)
	{
		if( abs(Yi - Ys )   > m_cStepY || //Y 
	        abs(Ui - Us )   > m_cStepU || //CB 
	        abs(Vi - Vs )   > m_cStepV )  //CR 
        Count ++;
     
		if( Count > m_lYUVSampleSize ) break ;

	}

	if( Count > m_lYUVSampleSize ) 
	{
		memcpy(m_pYUVSample , m_pYUV , m_lYUVSize );
		return TRUE ;
	}

	return  FALSE ;
}

