////////////////////////////////////////////////////////////////////////
//	Define the Video Motion Detection Dll which wrap CWnDrv Multi Thread Class   
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

CTypedPtrList<CObList,CWnDrv*>  m_threadList; 

WORD m_wHWCurCardNum =0; 

//++++++++++++++++++++++++++++++++++++++++
//Get Current Video Capture Card Number
//++++++++++++++++++++++++++++++++++++++++
WORD WnWd_GetCurrentCard() 
{
	return m_wHWCurCardNum ; 
}

//++++++++++++++++++++++++++++++++++++++++
//Set Current Video Capture Card Number
//++++++++++++++++++++++++++++++++++++++++
void WnWd_SetCurrentCard(WORD wHWCardNo) 
{
	m_wHWCurCardNum = wHWCardNo ;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Init one Video Capture Card Video Motion Detection thread 
//with the Card Number and the Real Time Video Frame Image Information
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void WnWd_CreateHWDrv(CWnd* pWnd ,
	                  WORD  wHWCardNo ,
					  BYTE* lpImage,
					  BITMAPINFO* lpDIBHdr)
{
	CWnDrv* pThread;

	pThread = new CWnDrv(pWnd,
							 wHWCardNo,
							 lpImage ,
							 lpDIBHdr);

	if (pThread == NULL) return;

	ASSERT_VALID(pThread);

	// Create Thread in a suspended state so we can set the Priority 
	// before it starts getting away from us
	if (!pThread->CreateThread(CREATE_SUSPENDED))
	{
		delete pThread;
		return;
	}

	// since everything is successful, add the thread to our list
	m_threadList.AddTail(pThread);

}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Start Current Video Capture Card Video Motion Detection thread
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void WnWd_StartHWDrv()
{
	CWnDrv* pThread ;
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		pThread = m_threadList.GetNext(pos);
		if( pThread->m_wHWCurrentCardNum ==  m_wHWCurCardNum)
		{
			pThread->m_DatProcMilliSecond = 80;  
			pThread->wnwd_IsSetting();
		    pThread->ResumeThread();
		}
	}

}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Stop Current Video Capture Card Video Motion Detection thread
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void WnWd_SuspendHWDrv()
{
	CWnDrv* pThread ;
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		pThread = m_threadList.GetNext(pos);
		if( pThread->m_wHWCurrentCardNum ==  m_wHWCurCardNum)
		{
		    pThread->SuspendThread();
		}
	}

}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Delete all Video Capture Card Video Motion Detection threads
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void WnWd_KillHWDrv() 
{
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		CWnDrv* pThread = m_threadList.GetNext(pos);
		pThread->SuspendThread();
		pThread->KillThread();
	}

	while (!m_threadList.IsEmpty())
	{
		CWnDrv* pThread = m_threadList.RemoveHead();
		delete pThread;
	}
	
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Save Current Video Capture Card Image to a  BMP File
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BOOL WnWd_SaveBMP(char *FileName)
{
	CWnDrv* pThread ;
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		pThread = m_threadList.GetNext(pos);
		if( pThread->m_wHWCurrentCardNum ==  m_wHWCurCardNum)
		{
		return pThread->wnwd_SaveBMP(FileName);
		}	
	}
	return FALSE ;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Set Current Video Capture Card Image X,Y Direction Sample point Step Number
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void WnWd_SetCmpStepXY( WORD wStepX ,WORD wStepY )
{
	CWnDrv* pThread ;
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		pThread = m_threadList.GetNext(pos);
		if( pThread->m_wHWCurrentCardNum ==  m_wHWCurCardNum)
		{
	    pThread->wnwd_SetCmpStepXY( wStepX , wStepY ); break ;
		}	
	}

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Set Current Video Capture Card Image YUV Comparison value
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void WnWd_SetCmpStepYUV(BYTE cStepY ,BYTE cStepU,BYTE cStepV )
{
	CWnDrv* pThread ;
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		pThread = m_threadList.GetNext(pos);
		if( pThread->m_wHWCurrentCardNum ==  m_wHWCurCardNum)
		{
	    pThread->wnwd_SetCmpStepYUV( cStepY , cStepU, cStepV ); break ;
		}	
	}

}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Set Current Video Capture Card Image Comparison Percent Number 
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void WnWd_SetCmpPercent( double dPercent )
{
	CWnDrv* pThread ;
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		pThread = m_threadList.GetNext(pos);
		if( pThread->m_wHWCurrentCardNum ==  m_wHWCurCardNum)
		{
	    pThread->wnwd_SetCmpPercent( dPercent ); break ;
		}	
	}

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Set Current Video Capture Card Video Motion Detection Saving Status 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
void WnWd_IsSave( BOOL bIsSave )
{
	CWnDrv* pThread ;
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		pThread = m_threadList.GetNext(pos);
		if( pThread->m_wHWCurrentCardNum ==  m_wHWCurCardNum)
		{
	    pThread->wnwd_IsSave( bIsSave ); break ;
		}	
	}

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Set Current Video Capture Card Video Motion Detection Warning Status
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  
void WnWd_IsWarn(BOOL bIsWarn)
{
	CWnDrv* pThread ;
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		pThread = m_threadList.GetNext(pos);
		if( pThread->m_wHWCurrentCardNum ==  m_wHWCurCardNum)
		{
	    pThread->wnwd_IsWarn( bIsWarn ); break ;
		}	
	}

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Set Current Video Capture Card Video Motion Detection Parameter  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void WnWd_IsSetting( )
{
	CWnDrv* pThread ;
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		pThread = m_threadList.GetNext(pos);
		if( pThread->m_wHWCurrentCardNum ==  m_wHWCurCardNum)
		{
	    pThread->wnwd_IsSetting(  ); break ;
		}	
	}

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Get Current Video Capture Card Video Motion Detection Status
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BOOL WnWd_GetCmpStatus()
{
	CWnDrv* pThread ;
	for (POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		pThread = m_threadList.GetNext(pos);
		if( pThread->m_wHWCurrentCardNum ==  m_wHWCurCardNum)
		{
	    return pThread->wnwd_GetCmpStatus( ); 
		}	
	}
	return FALSE ;
}

